<?php
require_once 'PHPUnit2/Framework/TestCase.php';
include_once "Zend.php";

class Config_IniTest extends PHPUnit2_Framework_TestCase
{
    protected $_iniFileConfig;
    protected $_iniFileNested;

    function __construct()
    {
        Zend::loadClass('Akrabat_Config_Ini');
        $this->_iniFileConfig = dirname(__FILE__).'/_files/config.ini';
    }

    function setUp()
    {
    }

    function tearDown()
    {
    }

    function testLoadSingleSection()
    {
        $array = Akrabat_Config_Ini::load($this->_iniFileConfig, 'all');

        $this->assertEquals('all', $array['hostname']);
        $this->assertEquals('live', $array['db']['name']);
        $this->assertEquals('multi', $array['one']['two']['three']);
        $this->assertNull(@$array['nonexistant']); // property doesn't exist
    }

    function testSectionInclude()
    {
        $array = Akrabat_Config_Ini::load($this->_iniFileConfig, 'staging');

        $this->assertEquals('', $array['debug']); // only in staging
        $this->assertEquals('thisname', $array['name']); // only in all
        $this->assertEquals('username', $array['db']['user']); // only in all (nested version)
        $this->assertEquals('staging', $array['hostname']); // inherited
        $this->assertEquals('dbstaging', $array['db']['name']); //inherited
    }

    function testMultiDepthExtends()
    {
        $array = Akrabat_Config_Ini::load($this->_iniFileConfig, 'other_staging');

        $this->assertEquals('otherStaging', $array['only_in']); // only in other_staging
        $this->assertEquals('', $array['debug']); // 1 level down: only in staging
        $this->assertEquals('thisname', $array['name']); //  2 levels down: only in all
        $this->assertEquals('username', $array['db']['user']); // 2 levels down: only in all (nested version)
        $this->assertEquals('staging', $array['hostname']); // inherited from two to one
        $this->assertEquals('dbstaging', $array['db']['name']); //inherited from two to one
        $this->assertEquals('anotherpwd', $array['db']['pass']); //inherited from two to other_staging
    }

    function testErrorNoInitialSection()
    {
        try {
            $array = @Akrabat_Config_Ini::load($this->_iniFileConfig);
        }
        catch (Exception $expected) {
            $this->assertContains('Section is not set', $expected->getMessage());
            return;
        }

        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorNoExtendsSection()
    {
        try {
            $array = Akrabat_Config_Ini::load($this->_iniFileConfig, 'extendserror');
        }
        catch (Exception $expected) {
            $this->assertContains('cannot be found', $expected->getMessage());
            return;
        }

        $this->fail('An expected Exception has not been raised.');
    }

}

?>